@@ -3,9 +3,13 @@ import os
 
 from functools import partial
 
+from dbnd._core.errors import DatabandError
 from dbnd_airflow.tracking.conf_operations import fix_keys, flat_conf
-from dbnd_airflow.tracking.config import TrackingConfig
-from dbnd_airflow.tracking.dbnd_airflow_conf import get_airflow_conf
+from dbnd_airflow.tracking.config import AirflowTrackingConfig
+from dbnd_airflow.tracking.dbnd_airflow_conf import (
+    extract_airflow_tracking_conf,
+    get_airflow_conf,
+)
 from dbnd_airflow.utils import logger
 
 
@@ -16,12 +20,12 @@ def spark_env(key):
 add_spark_env_fields = partial(fix_keys, spark_env)
 
 
-def _filter_vars(key, bypass_dbnd, bypass_airflow, bypass_rest):
+def _filter_vars(key, bypass_dbnd=True, bypass_airflow=True, bypass_rest=True):
     if key.startswith("AIRFLOW_"):
-        return True if bypass_airflow else False
-    if key.startswith("DBND_"):
-        return True if bypass_dbnd else False
-    return True if bypass_rest else False
+        return bypass_airflow
+    elif key.startswith("DBND_"):
+        return bypass_dbnd
+    return bypass_rest
 
 
 def dbnd_tracking_env(
@@ -44,16 +48,16 @@ def dbnd_wrap_spark_environment(environment=None):
 
 
 def with_dbnd_tracking_spark_conf(user_dict):
-    dbnd_dict = get_dbnd_tracking_spark_conf_dict()
+    dbnd_dict = get_dbnd_tracking_spark_conf()
     user_dict.update(dbnd_dict)
     return user_dict
 
 
-def get_dbnd_tracking_spark_conf_dict(**kwargs):
-    return dict(add_spark_env_fields(get_airflow_conf(**kwargs)))
-
-
 def get_dbnd_tracking_spark_conf(**kwargs):
+    return add_spark_env_fields(get_airflow_conf(**kwargs))
+
+
+def get_dbnd_tracking_spark_flat_conf(**kwargs):
     return flat_conf(add_spark_env_fields(get_airflow_conf(**kwargs)))
 
 
@@ -79,9 +83,10 @@ def spark_submit_with_dbnd_tracking(command_as_list, dbnd_context=None):
         An updated spark-submit command including dbnd context.
 
     """
-
-    if not dbnd_context:
-        dbnd_context = get_dbnd_tracking_spark_conf()
+    if dbnd_context:
+        context = dbnd_context
+    else:
+        context = get_dbnd_tracking_spark_flat_conf()
 
     index = next(
         (
@@ -94,29 +99,36 @@ def spark_submit_with_dbnd_tracking(command_as_list, dbnd_context=None):
     if index == -1:
         raise Exception("Failed to find spark-submit in %s" % " ".join(command_as_list))
 
-    return command_as_list[0 : index + 1] + dbnd_context + command_as_list[index + 1 :]
+    return command_as_list[0 : index + 1] + context + command_as_list[index + 1 :]
 
 
 logger = logging.getLogger(__name__)
 
 
-def get_local_spark_java_agent_conf():
-    config = TrackingConfig()
-    agent_jar = config.local_dbnd_java_agent
+def get_spark_submit_java_agent_conf():
+    config = AirflowTrackingConfig()
+    agent_jar = config.spark_submit_dbnd_java_agent
+    if agent_jar is None:
+        logger.warning("You are not using the dbnd java agent")
+        return
     logger.debug("found agent_jar %s", agent_jar)
-    if agent_jar is None or not os.path.exists(agent_jar):
-        logger.warning("The wanted agents jar doesn't exists: %s", agent_jar)
+    if not os.path.exists(agent_jar):
+        logger.warning(
+            "The wanted dbnd java agent is not found: {agent_path}".format(
+                agent_path=agent_jar
+            )
+        )
         return
 
     return create_spark_java_agent_conf(agent_jar)
 
 
 def get_databricks_java_agent_conf():
-    config = TrackingConfig()
+    config = AirflowTrackingConfig()
     agent_jar = config.databricks_dbnd_java_agent
     logger.debug("found agent_jar %s", agent_jar)
     if agent_jar is None:
-        logger.warning("No agent jar found")
+        logger.warning("You are not using the dbnd java agent")
         return
 
     return create_spark_java_agent_conf(agent_jar)
